import io
import pandas as pd

from django.http import HttpResponse


class ExcelResponse(HttpResponse):

    def __init__(
        self,
        data: list[list[str]],
        columns: list[str],
    ):
        self.data = data
        self.columns = columns

    def excel_response(self, file_name: str, title: str) -> HttpResponse:
        data = self.data
        columns = self.columns

        df = pd.DataFrame(data, columns=columns)

        with io.BytesIO() as buffer:
            with pd.ExcelWriter(buffer, engine="openpyxl") as writer:
                df.to_excel(writer, index=False, sheet_name=title)
            buffer.seek(0)
            response = HttpResponse(
                buffer.getvalue(),
                content_type="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
            )
            response["Content-Disposition"] = (
                f'attachment; filename="{file_name}.xlsx"'
            )
            return response



from django.db import models
from excel_extract.processors import Processor
from excel_extract.response import ExcelResponse


class Excel:

    def __init__(
        self,
        model: models.Model,
        queryset: models.QuerySet,
        file_name: str = 'file_name',
        title: str = 'title',
        choices: dict[str, dict[str, str]] = None,
        exclude: list[str] = None,
        date_format: str = None,
        date_time_format: str = None,
        bool_true: str = None,
        bool_false: str = None,
    ) -> None:
        self.model = model
        self.queryset = queryset
        self.exclude = set(exclude or [])
        self.file_name = file_name
        self.title = title
        self.date_format = date_format
        self.date_time_format = date_time_format
        self.bool_true = bool_true or 'True'
        self.bool_false = bool_false or 'False'
        self.choices = choices or {}
        self.fields = [
            field
            for field in self.model._meta.get_fields()
            if not isinstance(
                field,
                (
                    models.ManyToOneRel,
                    models.ManyToManyRel,
                    models.OneToOneRel,
                ),
            )
            and not (field.many_to_many and field.auto_created)
            and field.name not in self.exclude
        ]

        self.processor = Processor(
            date_format=self.date_format,
            date_time_format=self.date_time_format,
            bool_true=self.bool_true,
            bool_false=self.bool_false,
            choices=self.choices,
            exclude=exclude,
        )

    def get_fields(self) -> list[str]:
        return [str(field.verbose_name) for field in self.fields]

    def get_data_frame(self) -> list[list[str]]:
        data = []

        for item in self.queryset:
            values = []

            for field in self.fields:
                value = getattr(item, field.name)

                if value is None:
                    value = '-'

                if isinstance(field, models.ManyToManyField):
                    processor = self.processor._process_many_to_many
                else:
                    processor = self.processor.field_processors.get(
                        type(field),
                        self.processor._process_choices,
                    )

                value = processor(field, value, item)

                values.append(value)
            data.append(values)

        return data

    def to_excel(self):
        excel_response = ExcelResponse(
            data=self.get_data_frame(),
            columns=self.get_fields(),
        )
        return excel_response.excel_response(
            file_name=self.file_name, title=self.title
        )



from django.db import models
from datetime import datetime


class Processor:
    def __init__(
        self,
        date_format: str = None,
        date_time_format: str = None,
        bool_true: str = None,
        bool_false: str = None,
        choices: dict = None,
        exclude: list[str] = None,
    ):
        self.date_format = date_format
        self.date_time_format = date_time_format
        self.bool_true = bool_true
        self.bool_false = bool_false
        self.exclude = set(exclude or [])
        self.choices = choices or {}
        self.field_processors = self._generate_field_processors()
        self.display_choices = self._display_choices()

    def _generate_field_processors(self):
        return {
            models.DateField: self._process_date,
            models.BooleanField: self._process_boolean,
            models.DateTimeField: self._process_datetime,
            models.ManyToManyField: self._process_many_to_many,
        }

    def _display_choices(self) -> set[str]:
        if not hasattr(self, 'model'):
            return set()

        display_fields = set()

        for field in self.model._meta.fields:
            display_method_name = f"get_{field.name}_display"
            if hasattr(self.model, display_method_name):
                display_fields.add(field.name)

        return display_fields

    def _process_date(
        self, field: models.Field, value: str, item: models.Model
    ) -> str:
        print(value)
        print(type(value))
        print(self.date_format)
        if value == '-':
            return value

        if self.date_format:
            if isinstance(value, str):
                value = datetime.strptime(value, self.date_format)
                return value.strftime(self.date_format)
            else:
                return value.strftime(self.date_format)

    def _process_datetime(
        self, field: models.Field, value: str, item: models.Model
    ) -> str:
        if self.date_time_format:
            return value.strftime(self.date_time_format)
        return value.strftime('%Y-%m-%d %H:%M:%S')

    def _process_boolean(
        self, field: models.Field, value: bool, item: models.Model
    ) -> str:
        if self.bool_true and self.bool_false:
            return self.bool_true if value else self.bool_false

    def _process_many_to_many(
        self, field: models.Field, value: str, item: models.Model
    ) -> str:
        if field.name in self.exclude:
            return value
        if not item.pk:
            return ''

        many_to_many = getattr(item, field.name).all()
        return ', '.join(str(related_item) for related_item in many_to_many)

    def _process_choices(
        self, field: models.Field, value: str, item: models.Model
    ) -> str:
        return (
            getattr(item, f'get_{field.name}_display')()
            if field.name in self.display_choices
            and field.name not in self.exclude
            else value
        )
